/*********************************************************************
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2013, Ryohei Ueda and JSK Lab
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the JSK Lab nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *********************************************************************/

#ifndef LIGHTWEIGHT_THROTTLE_NODELET_H_
#define LIGHTWEIGHT_THROTTLE_NODELET_H_

#include <boost/shared_ptr.hpp>
#include <nodelet/nodelet.h>

#include <dynamic_reconfigure/server.h>
#include <jsk_topic_tools/LightweightThrottleConfig.h>
#include <topic_tools/shape_shifter.h>

namespace jsk_topic_tools
{
  class LightweightThrottle : public nodelet::Nodelet
  {
  public:
    typedef LightweightThrottleConfig Config;
    virtual void onInit();
    virtual void inCallback(const boost::shared_ptr<topic_tools::ShapeShifter const>& msg);
    virtual void connectionCallback(
      const ros::SingleSubscriberPublisher& pub);
    virtual void configCallback(Config& config, uint32_t level);
  protected:
    typedef boost::shared_ptr<ros::Subscriber> SubscriberPtr;
    boost::mutex mutex_;
    boost::shared_ptr<dynamic_reconfigure::Server<Config> > srv_;
    SubscriberPtr sub_;
    ros::Publisher pub_;
    ros::TransportHints th_;
    bool advertised_;
    bool subscribing_;
    double update_rate_;
    ros::Time latest_stamp_;
    ros::NodeHandle pnh_;
  };
}

#endif

